_G.DodgeTrackerStandalone = {
	_mod_path = ModPath,
	_options_path = ModPath .. "menu/options.txt",
	_save_path = SavePath .. "dodgetrackerstandalone_settings.txt",
	_instance = ModInstance,
	localization_path = ModPath .. "localization/english.json",
	localization = {
		en = { --deprecated; do not use
			["dodgetracker_menu_mod_title"] = "Dodge Tracker Standalone",
			["dodgetracker_menu_set_enabled_title"] = "Master Enable",
			["dodgetracker_menu_set_enabled_desc"] = "Must be enabled for tracker to work",
			["dodgetracker_menu_set_hud_compatibility_mode_title"] = "HUD Mod Compatibility Mode",
			["dodgetracker_menu_set_hud_compatibility_mode_desc"] = "Enable this if encounter technical difficulties",
			["dodgetracker_menu_set_x_title"] = "Set X",
			["dodgetracker_menu_set_x_desc"] = "Sets horizontal position",
			["dodgetracker_menu_set_y_title"] = "Set Y",
			["dodgetracker_menu_set_y_desc"] = "Sets vertical position",
			["dodgetracker_menu_set_halign_title"] = "Set Horizontal Align",
			["dodgetracker_menu_set_halign_desc"] = "Set Horizontal Align",
			["dodgetracker_menu_set_valign_title"] = "Set Vertical Align",
			["dodgetracker_menu_set_valign_desc"] = "Set Vertical Align",
			["dodgetracker_menu_set_threshold_title"] = "Set Dodge Threshold",
			["dodgetracker_menu_set_threshold_desc"] = "Set Dodge Threshold",
			["dodgetracker_menu_set_alpha_title"] = "Set Alpha",
			["dodgetracker_menu_set_alpha_desc"] = "Sets Tracker Opacity",
			["dodgetracker_menu_set_scale_title"] = "Set Scale",
			["dodgetracker_menu_set_scale_desc"] = "Set Scale",
			["dodgetracker_menu_left"] = "Left",
			["dodgetracker_menu_right"] = "Right",
			["dodgetracker_menu_center"] = "Center",
			["dodgetracker_menu_top"] = "Top",
			["dodgetracker_menu_bottom"] = "Bottom",
			["dodgetracker_menu_hititwithawrench_title"] = "Hit it with a wrench",
			["dodgetracker_menu_hititwithawrench_desc"] = "Tracker not showing up? Click to 'reboot' the tracker!",
			["dodgetracker_menu_reboot_1_title"] = "Rebooted!",
			["dodgetracker_menu_reboot_1_desc"] = "If this doesn't work, please contact me on Discord or Steam.\nMy contact information is in the mod.txt file.\n\n-Offyerrocker",
			["dodgetracker_menu_reboot_2_title"] = "Uh...",
			["dodgetracker_menu_reboot_2_desc"] = "This doesn't really do anything in the menu, by the way.\nI dunno, I guess maybe you just wanted to push the button?"
		}
	},
	settings = {
		enabled = true,
		hud_compatibility_mode = false,
		x = 1,
		y = 1,
		valign = 1,
		halign = 1,
		threshold = 0,
		scale = 32,
		alpha = 1,
		debug = false
	},
	H_SCALE = 3,
	_hud_box = nil, --just for name reference
	_workspace = nil,
	_hud_parent = nil,
	_hud_panel = nil,
	_is_adjusting = false
}

function DodgeTrackerStandalone:log(...)
	if self.settings.debug then
		if Console then 
			Console:Log(...)
		else
			log(...)
		end
	end
end

function DodgeTrackerStandalone:IsEnabled()
	return self.settings.enabled
end

function DodgeTrackerStandalone:IsHUDCompatibilityModeEnabled()
	return self.settings.hud_compatibility_mode
end

function DodgeTrackerStandalone:GetPosition()
	return self.settings.x,self.settings.y
end

function DodgeTrackerStandalone:GetHorizontalAlign()
	local halign = self.settings.halign
	if halign == 1 then 
		return "left"
	elseif halign == 2 then 
		return "center"
	elseif halign == 3 then 
		return "right"
	end
	return "left"
end

function DodgeTrackerStandalone:GetVerticalAlign()
	local valign = self.settings.valign
	if valign == 1 then 
		return "top"
	elseif valign == 2 then 
		return "center"
	elseif valign == 3 then 
		return "bottom"
	end
	return "top"
end

function DodgeTrackerStandalone:GetThreshold()
	if self.settings.threshold < 0 then 
		return -math.huge
	end
	return self.settings.threshold / 100
end

function DodgeTrackerStandalone:GetScale()
	return self.settings.scale
end

function DodgeTrackerStandalone:GetAlpha()
	return self.settings.alpha
end

function DodgeTrackerStandalone:CreateHUDBox(hud,skip_layout)
	if self:IsHUDCompatibilityModeEnabled() or not (hud and alive(hud)) then 
		self._workspace = self._workspace or managers.gui_data:create_fullscreen_workspace()
		self._hud_parent = self._hud_parent or self._workspace:panel()
		hud = self._hud_parent
	end
	
	local HALIGN = self:GetHorizontalAlign()
	local VALIGN = self:GetVerticalAlign()
	local ALPHA = self:GetAlpha()
	local SCALE = self:GetScale()
	local RAW_X,RAW_Y = self:GetPosition()

	local W = SCALE
	local H = SCALE * (1 + (1 / self.H_SCALE))

	local dodge_tracker = hud:panel({
		name = "dodgetrackerstandalone_panel",
		halign = HALIGN,
		valign = VALIGN,
		w = W,
		h = H,
		x = RAW_X,
		y = RAW_Y,
		alpha = ALPHA
	})
	self._hud_box = dodge_tracker
	
	local icon_x,icon_y = unpack(tweak_data.skilltree.skills.jail_diet.icon_xy)
	local texture_rect = {icon_x * 80,icon_y * 80,80,80}

	local icon = dodge_tracker:bitmap({
		name = "icon",
		texture = "guis/textures/pd2/skilltree_2/icons_atlas_2",
		texture_rect = texture_rect,
		w = SCALE,
		h = SCALE,
--		blend_mode = "add",
		halign = "grow",
		valign = "grow",
		layer = 1	
	})
	
	local label = dodge_tracker:text({
		name = "label",
		font_size = SCALE / self.H_SCALE,
		font = tweak_data.hud_players.ammo_font,
		text = "0%",
		align = "center",
		halign = "grow",
		color = Color.white,
		layer = 2
	})
	label:set_font_scale(SCALE)
	if not skip_layout then
		self:LayoutHUDBox()
	end
	return dodge_tracker
end

function DodgeTrackerStandalone:ShowTracker()
	if self._hud_box and alive(self._hud_box) then 
		self._hud_box:show()
	end
end

function DodgeTrackerStandalone:HideTracker()
	if self._hud_box and alive(self._hud_box) then 
		self._hud_box:hide()
	end
end

function DodgeTrackerStandalone:LayoutHUDBox(set_adjusting)
	if set_adjusting ~= nil then 
		self._is_adjusting = set_adjusting
	end
	local box = self._hud_box or self:CreateHUDBox(self._hud_parent,true)
	if not (box and alive(box)) then 
		self:log("Could not create HUD!")
		return
	end
	local hud = box:parent()
	
	local SCALE = self:GetScale()
	local HALIGN = self:GetHorizontalAlign()
	local VALIGN = self:GetVerticalAlign()
		
	box:set_valign(VALIGN)
	box:set_halign(HALIGN)
	box:set_alpha(self:GetAlpha())
	
	box:set_w(SCALE)
	box:set_h(SCALE * (1 + (1 / self.H_SCALE)))
	
	
	local RAW_X,RAW_Y = self:GetPosition()

	
	local icon = box:child("icon")
	icon:set_size(SCALE,SCALE)
	icon:set_position(0,0)
	local label = box:child("label")
	label:set_font_size(SCALE / self.H_SCALE)
	label:set_position(0,icon:bottom() + (SCALE / 16))
	local _,_,_,text_h = label:text_rect()

	local W = SCALE
	local H = SCALE + text_h

	local x,y
	if HALIGN == "left" then 
		x = RAW_X
	elseif HALIGN == "center" then
		x = ((hud:w() - W) / 2) + RAW_X
	elseif HALIGN == "right" then 
		x = (hud:w() - W) - RAW_X
	end
	if VALIGN == "top" then 
		y = RAW_Y
	elseif VALIGN == "center" then
		y = ((hud:h() - H) / 2) + RAW_Y
	elseif VALIGN == "bottom" then 
		y = (hud:h() - H) - RAW_Y
	end
	box:set_position(x,y)
end

function DodgeTrackerStandalone:Update(t,dt)
	if not self._is_adjusting then 
		local player = managers.player and managers.player:local_player()
		if player and self:IsEnabled() then 
			local hud_box = self._hud_box
			if not (hud_box and alive(hud_box)) then 
				self:CreateHUDBox()
			end
			local dodge_amount = managers.player:skill_dodge_chance(player:movement():running(),player:movement():crouching(),player:movement():zipline_unit(),false,managers.blackmarket:_get_concealment_from_local_player()) + managers.player:body_armor_value("dodge")
			if dodge_amount > self:GetThreshold() then
				hud_box:child("label"):set_text(string.format("%d%%",dodge_amount * 100))
				hud_box:show()
			else
				hud_box:hide()
			end
		else
			self:HideTracker()
		end
	end
end

function DodgeTrackerStandalone:OnGameStarted()
	if self._hud_box and alive(self._hud_box) then 
		self._hud_box:parent():remove(self._hud_box)		
	end
	local playerhud
	if managers.hud then
		managers.hud:remove_updator("dodgetrackerstandalone_update")
		playerhud = managers.hud:script(PlayerBase.PLAYER_INFO_HUD_FULLSCREEN_PD2)
		managers.hud:add_updator("dodgetrackerstandalone_update",callback(DodgeTrackerStandalone,DodgeTrackerStandalone,"Update"))
	end
	DodgeTrackerStandalone:CreateHUDBox(playerhud and playerhud.panel)
end

Hooks:Add("BaseNetworkSessionOnLoadComplete","DodgeTrackerStandalone_OnLoadComplete",DodgeTrackerStandalone.OnGameStarted)

function DodgeTrackerStandalone:Load()
	local file = io.open(self._save_path,"r")
	if file then
		for setting,value in pairs(json.decode(file:read("*all"))) do
			self.settings[setting] = value
		end
	else
		self:Save()
	end
end

function DodgeTrackerStandalone:Save()
	local file = io.open(self._save_path,"w+")
	if file then
		file:write(json.encode(self.settings))
		file:close()
	end
end

Hooks:Add("LocalizationManagerPostInit", "DodgeTrackerStandalone_LocalizationManagerPostInit", function( loc )
	if not BeardLib then 
		loc:load_localization_file(DodgeTrackerStandalone.localization_path) --default (english)
--		loc:add_localized_strings(DodgeTrackerStandalone.localization.en)
	end
end)

Hooks:Add( "MenuManagerInitialize", "DodgeTrackerStandalone_MenuManagerInitialize", function(menu_manager)

	MenuCallbackHandler.callback_dodgetracker_set_enabled = function(self,item)
		local value = item:value() == "on"
		DodgeTrackerStandalone.settings.enabled = value
		DodgeTrackerStandalone:Save()
	end
	
	MenuCallbackHandler.callback_dodgetracker_set_hud_compatibility_mode = function(self,item)
		DodgeTrackerStandalone.settings.hud_compatibility_mode = item:value() == "on"
		DodgeTrackerStandalone:Save()
		DodgeTrackerStandalone:LayoutHUDBox(true)
	end
	
	MenuCallbackHandler.callback_dodgetracker_set_hud_x = function(self,item)
		DodgeTrackerStandalone.settings.x = tonumber(item:value())
		DodgeTrackerStandalone:Save()
		DodgeTrackerStandalone:LayoutHUDBox(true)
	end
	
	MenuCallbackHandler.callback_dodgetracker_set_hud_y = function(self,item)
		DodgeTrackerStandalone.settings.y = tonumber(item:value())
		DodgeTrackerStandalone:Save()
		DodgeTrackerStandalone:LayoutHUDBox(true)
	end
	
	MenuCallbackHandler.callback_dodgetracker_set_hud_halign = function(self,item)
		DodgeTrackerStandalone.settings.halign = tonumber(item:value())
		DodgeTrackerStandalone:Save()
		DodgeTrackerStandalone:LayoutHUDBox(true)
	end
	
	MenuCallbackHandler.callback_dodgetracker_set_hud_valign = function(self,item)
		DodgeTrackerStandalone.settings.valign = tonumber(item:value())
		DodgeTrackerStandalone:Save()
		DodgeTrackerStandalone:LayoutHUDBox(true)
	end
	
	MenuCallbackHandler.callback_dodgetracker_set_dodge_threshold = function(self,item)
		DodgeTrackerStandalone.settings.threshold = tonumber(item:value())
		DodgeTrackerStandalone:Save()
		DodgeTrackerStandalone:LayoutHUDBox(true)
	end
	
	MenuCallbackHandler.callback_dodgetracker_set_scale = function(self,item)
		DodgeTrackerStandalone.settings.scale = tonumber(item:value())
		DodgeTrackerStandalone:Save()
		DodgeTrackerStandalone:LayoutHUDBox(true)
	end
	MenuCallbackHandler.callback_dodgetracker_set_alpha = function(self,item)
		DodgeTrackerStandalone.settings.alpha = tonumber(item:value())
		DodgeTrackerStandalone:Save()
		DodgeTrackerStandalone:LayoutHUDBox(true)
	end
	
	MenuCallbackHandler.callback_dodgetracker_hititwithawrench = function(self)
		DodgeTrackerStandalone:OnGameStarted()
		local title,desc
		if managers.player then
			title = "dodgetracker_menu_reboot_1_title"
			desc = "dodgetracker_menu_reboot_1_desc"
		else
			title = "dodgetracker_menu_reboot_2_title"
			desc = "dodgetracker_menu_reboot_2_desc"
		end
		
		QuickMenu:new(managers.localization:text(title),managers.localization:text(desc),{
			{
				text = ":)",
				is_cancel_button = true,
				is_focused_button = true
			},
			{
				text = "fuck you offy!",
				callback = function()
					QuickMenu:new(":(","That wasn't very nice...",{
						{
							text = " ",
							is_cancel_button = true
						}
					},true)
				end
			}
		},true)
	end
	
	MenuCallbackHandler.callback_dodgetracker_close = function(self,item)
		DodgeTrackerStandalone._is_adjusting = false
		if not managers.player:local_player() then 
			DodgeTrackerStandalone:HideTracker()
		end
	end
	
	DodgeTrackerStandalone:Load()
	
	MenuHelper:LoadFromJsonFile(DodgeTrackerStandalone._options_path, DodgeTrackerStandalone, DodgeTrackerStandalone.settings)
end)
